#!/usr/bin/env bash

################################################################################
#
# This script will download the perl packages (nd their dependencies) needed for
# the sDOT installer
#
# Takes about 1 minute to run
#
################################################################################

shopt -s globstar
shopt -s extglob
shopt -s nullglob

overrides_dir=version_overrides

#List of modules to get from the mirror (currently copied from
#installer-packaging.sh). These will each be retrieved and scanned recursively
#for dependencies. Each package listed here will be downloaded along with its
#dependencies in a directory named Package_Name and will contain the .tar.gz
#files that make up the package and its dependencies
perl_modules=("Compress::Zlib" "Crypt::OpenSSL::RSA" "Crypt::X509"
              "Exception::Class" "IO::Compress::Base"
              "IO::Compress::Zlib::Constants" "IO::Socket::INET6"
              "JSON::Schema" "Log::Dispatch::FileRotate" "MIME::Base64"
              "Mojolicious" "Net::INET6Glue" "Path::Class"
              "UUID" "UUID::Random")

for module in "${perl_modules[@]}"; do
    #name the directory according to its package name
    module_dir=$(echo ${module//:/_} | tr -s '_')

    #calculate the dependencies for each package and save the tarball in the
    #previously calcualted directory name. Also save the depth-first list of the
    #dependencies so that we know what order to reinstall them
    mkdir -p ${module_dir}_tmp
    cpanm -L ${module_dir}_tmp --save-dists ${module_dir}_tmp --scandeps --mirror http://repomirror-rtp.eng.netapp.com/CPAN --mirror-only --format dists $module > ${module_dir}_tmp/deps.txt

    #need to massage the deps.txt file we created to remove the uneeded
    #author information
    sed -Ei 's|.+/(.+\.tar\.gz)|\1|g' ${module_dir}_tmp/deps.txt

    #the above creates a directory structure that is based on the /author/id/...
    #structure that is on cpan, we only care about the .tar.gz files that are
    #required for each package so we flatten the structure a little bit
    mkdir -p $module_dir
    cp ${module_dir}_tmp/**/*.tar.gz $module_dir
    cp ${module_dir}_tmp/deps.txt $module_dir
    rm -r ${module_dir}_tmp
done

#If there are any .tar.gz files in the version_overrides directory, we need to
#replace any of the modules we just got with those (probably for compatibility)
for override in $overrides_dir/*.tar.gz; do
    #get the generic package name (not including version number) to try to
    #match to the other directories
    generic=$(basename $override | sed -E 's|([^0-9]+).*(\.tar\.gz)|\1*\2|p')

    for to_replace in !($overrides_dir)/$generic; do
        echo "Replacing $to_replace with $override"
        cp $override $(dirname $to_replace)
        rm -f $to_replace

        #need to update the deps.txt as well
        sed -Ei "s|$(basename $to_replace)|$(basename $override)|" $(dirname $to_replace)/deps.txt
    done
done
